# Rhye's and Fall of Civilization - Dynamic resources

from CvPythonExtensions import *
import CvUtil
import PyHelpers
from Core import *
from RFCUtils import * # edead
from StoredData import data

from Events import handler


@handler("GameStart")
def setupOnGameStart():
	setup()
	
@handler("OnLoad")
def setupOnLoad():
	setup()

def setup():
	global dResources
	dResources = TileDict(dResourcesDict, year)
	
	global dSpawnResources
	dSpawnResources = TileDict(dSpawnResourcesDict)
	
	global dRemovedResources
	dRemovedResources = TileDict(dRemovedResourcesDict, year)
	
	global dPlotTypes
	dPlotTypes = TileDict(dPlotTypesDict, year)
	
	global dFeatures
	dFeatures = TileDict(dFeaturesDict, year)
	
	for tile in lNewfoundlandCapes:
		dFeatures[tile] = (700, iCape)
	
	global dRemovedFeatures
	dRemovedFeatures = TileDict(dRemovedFeaturesDict, year)
	
	for tile in lNewfoundlandCapes:
		dRemovedFeatures[tile] = 1500
		
	global dRoutes
	dRoutes = appenddict(dict((year(iYear), tiles) for iYear, tiles in dRoutesDict.items()))


### Constants ###

# initialise bonuses variables

lSilkRoute = [(85,48), (86,49), (87,48), (88,47), (89,46), (90,47), (90,45), (91,47), (91,45), (92,48), (93,48), (93,46), (94,47), (95,47), (96,47), (97,47), (98,47), (99,46)]
lNewfoundlandCapes = [(34, 52), (34, 53), (34, 54), (35, 52), (36, 52), (35, 55), (35, 56), (35, 57), (36, 51), (36, 58), (36, 59)]

# old world resources
constNativeDate = 610
# early columbian exchange
constColonizerDate = 1600
#plot changes date
constPlotChangeDate = 600

dResourcesDict = {
	(88, 37)  : (-1000, iHorse),   # Gujarat
	(78, 42)  : (-800,  iCopper),  # Assyria
	(88, 47)  : (-100,  iSilk),    # Silk Route
	(85, 46)  : (-100,  iSilk),    # Silk Route
	(108, 47) : (-50,   iPig),     # Hanseong
	(71, 34)  : (constNativeDate,   iIron),    # Egypt
	(72, 24)  : (constNativeDate,  iSugar),   # East Africa
	(70, 17)  : (constNativeDate,  iSugar),   # Zimbabwe
	(67, 11)  : (constNativeDate,  iSugar),   # South Africa
	(66, 23)  : (constNativeDate,  iBanana),  # Central Africa
	(64, 20)  : (constNativeDate,  iBanana),  # Central Africa
	(57, 46)  : (constNativeDate,  iWine),    # Savoy
	(57, 45)  : (constNativeDate,  iClam),    # Savoy
	(50, 44)  : (constNativeDate,  iIron),    # Portugal
	(96, 36)  : (constNativeDate,  iFish),    # Bengal
	(56, 54)  : (constNativeDate,  iFish),    # Amsterdam
	(57, 52)  : (constNativeDate,  iWheat),   # Amsterdam
	(58, 52)  : (constNativeDate,  iCow),     # Amsterdam
	(29, 52)  : (constColonizerDate,  iCow),     # Montreal
	(18, 53)  : (constColonizerDate,  iCow),     # Alberta
	(12, 52)  : (constColonizerDate,  iCow),     # British Columbia
	(28, 46)  : (constColonizerDate,  iCow),     # Washington area
	(30, 49)  : (constColonizerDate,  iCow),     # New York area
	(23, 42)  : (constColonizerDate,  iCow),     # El Paso
	(18, 46)  : (constColonizerDate,  iCow),     # Colorado
	(20, 45)  : (constColonizerDate,  iCow),     # Texas
	(37, 14)  : (constColonizerDate,  iCow),     # Argentina
	(33, 11)  : (constColonizerDate,  iCow),     # Argentina
	(35, 10)  : (constColonizerDate,  iCow),     # Pampas
	(24, 43)  : (constColonizerDate,  iCotton),  # near Florida
	(23, 45)  : (constColonizerDate,  iCotton),  # Louisiana
	(22, 44)  : (constColonizerDate,  iCotton),  # Louisiana
	(13, 45)  : (constColonizerDate,  iCotton),  # California
	(25, 49)  : (constColonizerDate,  iPig),     # Lakes
	(19, 51)  : (constColonizerDate,  iSheep),   # Canadian border
	(19, 48)  : (constColonizerDate,  iWheat),   # Midwest
	(20, 53)  : (constColonizerDate,  iWheat),   # Manitoba
	(22, 33)  : (constColonizerDate,  iBanana),  # Guatemala
	(27, 31)  : (constColonizerDate,  iBanana),  # Colombia
	(43, 23)  : (constColonizerDate,  iBanana),  # Brazil
	(39, 26)  : (constColonizerDate,  iBanana),  # Brazil
	(49, 44)  : (constColonizerDate,  iCorn),    # Galicia
	(54, 48)  : (constColonizerDate,  iCorn),    # France
	(67, 47)  : (constColonizerDate,  iCorn),    # Romania
	(106, 50) : (constColonizerDate,  iCorn),    # Manchuria
	(77, 52)  : (constColonizerDate,  iCorn),    # Caricyn
	(92, 35)  : (constColonizerDate,  iSpices),  # Deccan
	(16, 54)  : (constColonizerDate,  iHorse),   # Alberta
	(26, 45)  : (constColonizerDate,  iHorse),   # Washington area
	(21, 48)  : (constColonizerDate,  iHorse),   # Midwest
	(19, 45)  : (constColonizerDate,  iHorse),   # Texas
	(17, 42)  : (constColonizerDate,  iHorse),   # Mexico
	(40, 25)  : (constColonizerDate,  iHorse),   # Brazil
	(33, 10)  : (constColonizerDate,  iHorse),   # Buenos Aires area
	(32, 8)   : (constColonizerDate,  iHorse),   # Pampas
	(30, 30)  : (constColonizerDate,  iHorse),   # Venezuela
	(27, 36)  : (constColonizerDate,  iSugar),   # Caribbean
	(39, 25)  : (constColonizerDate,  iSugar),   # Brazil
	(37, 20)  : (constColonizerDate,  iSugar),   # inner Brazil
	(29, 37)  : (constColonizerDate,  iSugar),   # Hispaniola
	(104, 52) : (constColonizerDate,  iCorn),    # Manchuria
	(89, 36)  : (constColonizerDate,  iCorn),    # India
	(38, 18)  : (constColonizerDate,  iCoffee),  # Brazil
	(39, 20)  : (constColonizerDate,  iCoffee),  # Brazil
	(38, 22)  : (constColonizerDate,  iCoffee),  # Brazil
	(27, 30)  : (constColonizerDate,  iCoffee),  # Colombia
	(29, 30)  : (constColonizerDate,  iCoffee),  # Colombia
	(26, 27)  : (constColonizerDate,  iCoffee),  # Colombia
	(104, 25) : (constColonizerDate,  iCoffee),  # Java
	(67, 44)  : (constColonizerDate,  iTobacco), # Turkey
	(39, 16)  : (constNativeDate,  iFish),    # Brazil
	(70, 59)  : (constNativeDate,  iDeer),    # St Petersburg
	(12, 45)  : (constColonizerDate,  iWine),    # California
	(31, 10)  : (constColonizerDate,  iWine),    # Andes
	(113, 11) : (constColonizerDate,  iWine),    # Barossa Valley
	(114, 11) : (constColonizerDate,  iSheep),   # Australia
	(116, 13) : (constColonizerDate,  iSheep),   # Australia
	(121, 6)  : (constColonizerDate,  iSheep),   # New Zealand
	(58, 47)  : (constColonizerDate,  iRice),    # Vercelli
	(12, 49)  : (constColonizerDate,  iRice),    # California
	(11, 45)  : (constNativeDate,  iFish),    # California
	(87, 35)  : (constNativeDate,  iFish),    # Mumbai
	(115, 52) : (constNativeDate,  iCow),     # Hokkaido
	(1, 38)   : (constNativeDate,  iSugar),   # Hawaii
	(5, 36)   : (constNativeDate,  iBanana),  # Hawaii
	(108, 18) : (constNativeDate,  iCamel),   # Australia
	# conditionals
	(90, 28) : (constNativeDate, iFish),
	(95, 43) : (constNativeDate, iWheat),
	(97, 44) : (constNativeDate, iHorse),
	(78, 51) : (constNativeDate, iSilk),
	(61, 22) : (constNativeDate, iCotton),
	(63, 19) : (constNativeDate, iIvory),
	(61, 24) : (constNativeDate, iIvory),
	(17, 41) : (constColonizerDate, iHorse),
	(16, 42) : (constNativeDate, iIron),
	(28, 31) : (constNativeDate, iIron),
	(31, 10) : (constColonizerDate, iWine),
	(31, 6)  : (constColonizerDate, iSheep),
	(32, 11) : (constNativeDate, iIron),
	(36, 18) : (constColonizerDate, iCorn),
	(42, 18) : (constNativeDate, iFish),														
	#extra
	(40, 19)  : (constNativeDate,  iIron),   	# Rio
	(41, 21)  : (constNativeDate,  iAluminium),  # Bahia
	(61, 21)  : (constNativeDate,  iCotton),  # congo
	(100, 29) : (constNativeDate, iPearls),	#singapore
	(71, 35) : (constNativeDate, iPearls),
	(113, 43) : (constNativeDate, iAluminium),
	(110, 55) : (constNativeDate, iGems),
	(91, 54) : (constNativeDate, iCow),
	(69, 57) : (constNativeDate, iCow),
	(62, 54) : (constNativeDate, iCow),
	(61, 49) : (constNativeDate, iWine),
	(62, 52) : (constNativeDate, iDeer),
	(54, 50) : (constNativeDate, iWheat),
	(52, 55) : (constNativeDate, iPig),
	(53, 53) : (constNativeDate, iHorse),
	(51, 53) : (constNativeDate, iCopper),
	(30, 48) : (constNativeDate, iCopper),	# new york
	(118, 48) : (constNativeDate, iWhales),
	(116, 44) : (constNativeDate, iWhales),
	(106, 46) : (constNativeDate, iCrab),
	(98, 42) : (constNativeDate, iCow),
	(53, 41) : (constNativeDate, iCopper),
	(49, 53) : (constNativeDate, iCrab),
	(56, 55) : (constNativeDate, iWhales),
	(70, 41) : (constNativeDate, iClam),
	(70, 49) : (constNativeDate, iFish),
	(67, 56) : (constNativeDate, iFur),  # novgorod fur
	(66, 15) : (constColonizerDate, iCorn),
	(27, 27) : (constNativeDate, iBanana),
	(54, 45) : (constNativeDate, iHorse),
	(61, 40) : (constNativeDate, iMarble),
	(117, 49) : (constNativeDate, iCrab),
	(114, 44) : (constNativeDate, iRice),
	(115, 45) : (constNativeDate, iIncense),
	(106, 44) : (constNativeDate, iCopper),
	(79, 59) : (constNativeDate, iDeer),
	(82, 55) : (constNativeDate, iWheat),
	(108, 38) : (constNativeDate, iMarble),
	(69, 36) : (constNativeDate, iSugar),
	(101, 30) : (constNativeDate, iGold),
	(108, 28) : (constNativeDate, iGold),
	(101, 26) : (constNativeDate, iCoffee),
	(110, 35) : (constNativeDate, iFish),
	(30, 39) : (constNativeDate, iCoffee),
	(8, 48) : (constNativeDate, iWhales),	# san francisco
	(59, 25) : (constNativeDate, iFish),
	(63, 40) : (constNativeDate, iClam),
	(60, 41) : (constNativeDate, iFish),
	(56, 50) : (constNativeDate, iHorse),
	(70, 58) : (constNativeDate, iFish),
	(54, 54) : (constNativeDate, iFish),
	(104, 38) : (constNativeDate, iPearls),
	(106, 39) : (constNativeDate, iPearls),
	(103, 24) : (constNativeDate, iClam),
	(107, 26) : (constNativeDate, iCrab),
	(107, 59) : (constNativeDate, iDeer),
	(107, 61) : (constNativeDate, iOil), # Siberia
	(27, 37) : (constNativeDate, iPearls),
	(31, 38) : (constNativeDate, iPearls),
	(34, 34) : (constNativeDate, iPearls),
	(21, 41) : (constNativeDate, iCrab),
	(32, 51) : (constNativeDate, iIron),
	(33, 51) : (constNativeDate, iFur),
	(29, 49) : (constNativeDate, iIron),
	(78, 49) : (constNativeDate, iPearls),
	(43, 21) : (constNativeDate, iFish),
	(47, 42) : (constNativeDate, iWhales),
	(116, 46) : (constNativeDate, iSilk),
	(116, 48) : (constNativeDate, iStone),
	(12, 45)  : (constNativeDate,  iGold),    # California
	(25, 42)  : (constNativeDate,  iDye),    # Florida
	(94, 50)  : (constNativeDate,  iCamel),    # Urumqi
	(52, 57)  : (constNativeDate,  iCoal),    # Manchester
	(56, 48)  : (constNativeDate,  iCoal),    # Lyon
	(57, 50)  : (constNativeDate,  iWine),    # Champagne
	(81, 53)  : (constNativeDate,  iCoal),    # Ural
	(69, 51)  : (constNativeDate,  iPig),    # Kiev
	(72, 52)  : (constNativeDate,  iCoal),    # Kharkiv
	(81, 47)  : (constNativeDate,  iOil),    # Turkmenistan
	(92, 60)  : (constNativeDate,  iOil),    # Tomsk
	(30, 20)  : (constNativeDate,  iTobacco),    # Bolivia coca
	(34, 17)  : (constColonizerDate,  iTea),    # Paraguay mate
	(61, 57)  : (constNativeDate,  iCrab),    # Denmark
	(14, 44)  : (constColonizerDate,  iWheat),    # Phoenix
	(29, 44)  : (constNativeDate,  iWhales),    # Washington
	(84, 36)  : (constNativeDate,  iCamel),    # karachi
	(85, 35)  : (constNativeDate,  iFish),    # karachi
	(29, 32)  : (constNativeDate,  iCorn),   # Caracas
	(29, 34)  : (constNativeDate,  iFish),    # Maracaibo
	(103, 41)  : (constNativeDate,  iRice),    # Wuhan
	(103, 42)  : (constNativeDate,  iGold),    # Wuhan
	(111, 42)  : (constNativeDate,  iCopper),    # Nagasaki
	(11, 43)  : (constNativeDate,  iFish),    # San Diego
	(10, 51)  : (constNativeDate,  iCrab),    # Seattle
	(116, 51)  : (constNativeDate,  iCoal),    # Sapporo
	(62, 43)  : (constNativeDate,  iMarble),    # Naples
	(51, 38)  : (constNativeDate,  iCamel),    # Marrakesh
	(108, 32)  : (constNativeDate,  iCopper),    # Manila Swap
	(108, 35)  : (constNativeDate,  iGold),    # Manila Swap
	(81, 41)  : (constNativeDate,  iSheep),    # Tehran
	(68, 50)  : (constNativeDate,  iCoal),    # Odessa
	(109, 31)  : (constNativeDate,  iSpices),    # Davao
	(31, 54)  : (constNativeDate,  iDeer),    # Montreal
	(26, 52)  : (constNativeDate,  iDeer),    # Toronto
	(111, 44)  : (constNativeDate,  iPearls),    # kyoto
	(93, 47)  : (constNativeDate,  iOil),    # china oil in xinjiang
	(98, 47)  : (constNativeDate,  iSheep),    # baotou
	(108, 51)  : (constNativeDate,  iAluminium),    # manchuria
	(65, 55)  : (constNativeDate,  iGold),    # baltic amber
	(72, 54)  : (constNativeDate,  iPig),    # moscow food
	(79, 44)  : (constNativeDate,  iFish),	# caspian food
	(73, 48)  : (constNativeDate,  iPearls),	# azov pearls
	(67, 51)  : (constNativeDate,  iDye),	# warsaw
	(65, 53)  : (constNativeDate,  iHorse),	# warsaw
	(32, 21)  : (constNativeDate,  iBanana),	# la paz banana
	(60, 55)  : (constNativeDate,  iClam),	# denmark and hamburg
	(10, 43)  : (constNativeDate,  iWhales),	# san diego
	(56, 60)  : (constNativeDate,  iWhales),	# west norway
	(27, 38)  : (constNativeDate,  iCopper),	# copper cuba
	(36, 49)  : (constNativeDate,  iWhales),	# halifax
	(29, 55)  : (constNativeDate,  iFur),	# quebec
	(54, 51)  : (constNativeDate,  iDye),	# north france
	(32, 49)  : (constNativeDate,  iFish),	# boston
	(32, 48)  : (constNativeDate,  iClam),	# boston
	(111, 10) : (constNativeDate,  iWhales),	# adelaide
	(53, 61)  : (constNativeDate,  iWhales),	# plymouth
	(66, 59)  : (constNativeDate,  iFur),	# helsinki
	(24, 22)  : (constNativeDate,  iWhales),	# whales lima
	(71, 62)  : (constNativeDate,  iWhales),	# whales arhangelsk
	(114, 57) : (constNativeDate,  iWhales),	# whales magadan
	(121, 54) : (constNativeDate,  iWhales),	# whales magadan
	(48, 52)  : (constNativeDate,  iWhales),	# whales plymouth
	(49, 49)  : (constNativeDate,  iWhales),	# whales brest
	(48, 47)  : (constNativeDate,  iWhales),	# whales cantabria
	(27, 7)   : (constNativeDate,  iWhales),	# whales chile
	(68, 48)  : (constNativeDate,  iWine),	# wine moldavia
	(76, 46)  : (constNativeDate,  iWine),	# wine georgia
	(121, 10) : (constNativeDate,  iCrab),	# crab new zealand
	(32, 53)  : (constNativeDate,  iFish),	# montreal
	(107, 17) : (constNativeDate,  iGold),	# australia
	(70, 54)  : (constNativeDate,  iIron),	# kiev
	(69, 60)  : (constNativeDate,  iCopper),	# st petersburg
	(11, 50)  : (constNativeDate,  iDeer),	# san francisco
	(51, 45)  : (constNativeDate,  iPig),		# madrid
	(73, 50)  : (constNativeDate,  iCoal),		# rostov
	(107, 45) : (constNativeDate,  iFish),		# shanghai fish
	(109, 43) : (constNativeDate,  iWhales),		# shanghai whales
	(77, 55) : (constNativeDate,  iHorse),		# Kazan horses
	(91, 50) : (constNativeDate,  iCotton),		# Urumqi cotton
	(80, 29) : (constNativeDate,  iSpices),		# Salalah
	(119, 11) : (constNativeDate,  iWhales),		# sydney
	(43, 17) : (constNativeDate,  iWhales),		# 
	(36, 56) : (constNativeDate,  iWhales),		# 
	(105, 20) : (constNativeDate,  iOil),		# australia
	(32, 50) : (constNativeDate,  iDeer),		# boston
	
	(57, 43)  : (constNativeDate,  iPearls),	# mediteranean
	(68, 47)  : (constNativeDate,  iPearls),	# mediteranean
	(61, 42)  : (constNativeDate,  iPearls),	# mediteranean
	(75, 30)  : (constNativeDate,  iPearls),	# mediteranean
	(80, 35)  : (constNativeDate,  iPearls),	# mediteranean
	
	(51, 60)  : (constNativeDate,  iAluminium),	# aluminum scotland (strategy + zinc)
	(32, 3)  : (constNativeDate,  iFur),		# patagonia
	
	(66, 49)  : (constNativeDate,  iGold),	# carpathian gold instead of iron
	(61, 50)  : (constNativeDate,  iSilver),	# austria
	(64, 56)  : (constNativeDate,  iPearls),	# baltic pearls (fish leads to too much pop)
	(102, 37)  : (constNativeDate, iCrab),	# hanoi
	(103, 36)  : (constNativeDate, iPearls),	# tonkin
	(47, 36)  : (1400, iSugar),	# practically whole economy of canaries
	(53, 29)  : (constNativeDate, iGold),	# mali gold
	(59, 26)  : (constNativeDate, iOil),		# lagos oil
	(59, 27)  : (constNativeDate,  iGold),  # lagos gold
	
	(66, 21)  : (constNativeDate,  iCopper),  # congo
	
	(15, 48)  : (constColonizerDate,  iCow),  # las vegas
	(9, 47)  : (constNativeDate,  iClam),  # los angeles
	
	(29, 46)  : (constNativeDate,  iFish),  # philadelphia
	(31, 46)  : (constNativeDate,  iWhales),  # new york
	(29, 12)  : (constNativeDate,  iCopper),  # santiago
	(29, 10)  : (constNativeDate,  iDeer),  # santiago
	(29, 9)  : (constColonizerDate,  iWine),  # santiago
	
	(33, 14)  : (constColonizerDate,  iTea),  # asuncion mate tea
	
	(21, 58)  : (constNativeDate,  iGems),  # canada gems
	(15, 60)  : (constNativeDate,  iGems),  # canada gems
	
	(42, 27)  : (constNativeDate,  iWhales),  # fortaleza
	
	(59, 30) : (constNativeDate,  iCamel),   # kano
	(110, 43) : (constNativeDate,  iFish),   # fukuoka
	(82, 62) : (constNativeDate,  iCoal),   # novy port
	
	(111, 55) : (constNativeDate,  iDeer),   # khabarovsk
	(109, 59) : (constNativeDate,  iOil),   # Okhotsk
	
	(106, 55) : (constNativeDate,  iFur),   # Manchuria
	(115, 61) : (constNativeDate,  iFur),   # Magadan
	(97, 62) : (constNativeDate,  iFur),   # Khatanga
	
	(113, 17) : (constNativeDate,  iCopper),   # Australia
	(19, 34) : (constNativeDate,  iStone),   # Puebla, Maya
	
	(58, 35) : (constNativeDate,  iStone),   # uraniun lybia
	
	(31, 26) : (constNativeDate,  iBanana),   # amazon
	(103, 23) : (constNativeDate,  iWhales),   # jakarta
	(104, 39) : (constNativeDate,  iStone),   # guangzhou
}

# TODO: should be handled by Rise and Fall
# TODO: Rise and Fall needs generic function canSpawn for normal civs and rebirths
dSpawnResourcesDict = {
	(90, 28) : (iTamils,    iFish),
	(95, 43) : (iTibet,     iWheat),
	(97, 44) : (iTibet,     iHorse),
	(78, 51) : (iMongols,   iSilk),
	(61, 22) : (iCongo,     iCotton),
	(63, 19) : (iCongo,     iIvory),
	(61, 24) : (iCongo,     iIvory),
	(17, 41) : (iMexico,    iHorse),
	(16, 42) : (iMexico,    iIron),
	(28, 31) : (iColombia,  iIron),
	(31, 10) : (iArgentina, iWine),
	(31, 6)  : (iArgentina, iSheep),
	(32, 11) : (iArgentina, iIron),
	(36, 18) : (iBrazil,    iCorn),
	(42, 18) : (iBrazil,    iFish),
}

dRemovedResourcesDict = {
	(51, 36) : 550, # Ivory in Morocco
	(58, 37) : 550, # Ivory in Tunisia
	(87, 49) : 1100, # Sheep near Orduqent
	(89, 51) : 1100, # Camel near Orduqent
	(63, 66) : constNativeDate, # north norway whales
	(45, 61) : constNativeDate, # iceland fish
	(51, 62) : constNativeDate, # scotland whales
	(30, 53) : constNativeDate, # quebec fur
	(113, 7) : constNativeDate, # tasmania whales
	(49, 57) : constNativeDate, # ireland horse
	(23, 22) : constNativeDate, # whales quito
	(26, 19) : constNativeDate, # whales lima
	(119, 52) : constNativeDate, # whales kamchatka
	(121, 8) : constNativeDate, # crab new zealand
	(69, 50) : constNativeDate, # iron ukraine
	(67, 57) : constNativeDate, # cow estonia
	(68, 60) : constNativeDate, # copper helsinki
	(50, 47) : constNativeDate, # crab cantabria
	(11, 49) : constNativeDate, # deer san francisco
	(66, 53) : constNativeDate, # horse warsaw
	(74, 50) : constNativeDate, # coal rostov
	(92, 50) : constNativeDate, # cotton urumqi
	(22, 38) : constNativeDate, # crab maya
	(38, 55) : constNativeDate, 
	(65, 59) : constNativeDate, 
	(102, 19) : constNativeDate, 
	(101, 17) : constNativeDate, 
	(61, 22) : constNativeDate, #congo cotton
	(53, 28) : constNativeDate, #mali gold
	(60, 26) : constNativeDate, #lagos oil
	(66, 20) : constNativeDate, #congo copper
	(10, 47) : constNativeDate, #clam
	(30, 46) : constNativeDate, #fish
	
	(17, 62) : constNativeDate, #gems canada
	(19, 61) : constNativeDate, #gems canada
	(39, 29) : constNativeDate, #whales macapa
	(106, 62) : constNativeDate, #sakha fur
	(107, 61) : constNativeDate, #sakha oil
	
	(105, 55) : constNativeDate, #manchuria fur
	(113, 18) : constNativeDate, #australia copper
	
	(19, 35) : constNativeDate, #stone maya
	(59, 34) : constNativeDate, #uranium lybia
	
	(29, 9) : constNativeDate,	#santiago deer
	(28, 3) : constNativeDate, #tierra del fuego						   
}

dRoutesDict = {
	-200 : lSilkRoute,
	-100 : [(88, 47)],
}

dSpawnRoutes = {
	iMongols : [(101, 48), (100, 49), (100, 50), (99, 50)],
}

# there must be stuff like this elsewhere, maybe barbs?
dPlotTypesDict = {
	(88, 47) : (-100, PlotTypes.PLOT_HILLS),
	
	(25, 25) : (constPlotChangeDate, PlotTypes.PLOT_HILLS),	#various
	(27, 29) : (constPlotChangeDate, PlotTypes.PLOT_HILLS),
	(31, 20) : (constPlotChangeDate, PlotTypes.PLOT_HILLS),
	(112, 26) : (constPlotChangeDate, PlotTypes.PLOT_HILLS),
	(28, 32) : (constPlotChangeDate, PlotTypes.PLOT_HILLS),
	(108, 35) : (constPlotChangeDate, PlotTypes.PLOT_HILLS),
	(115, 51) : (constPlotChangeDate, PlotTypes.PLOT_HILLS),
	(114, 51) : (constPlotChangeDate, PlotTypes.PLOT_PEAK),
	(108, 36) : (constPlotChangeDate, PlotTypes.PLOT_PEAK),
	(115, 7) : (constPlotChangeDate, PlotTypes.PLOT_PEAK),
	(115, 8) : (constPlotChangeDate, PlotTypes.PLOT_HILLS),
	(23, 50) : (constPlotChangeDate, PlotTypes.PLOT_HILLS),
	(107, 44) : (constPlotChangeDate, PlotTypes.PLOT_LAND),
	(72, 23) : (constPlotChangeDate, PlotTypes.PLOT_HILLS),	#nairobi
	(71, 23) : (constPlotChangeDate, PlotTypes.PLOT_PEAK),	#nairobi
	(47, 63) : (constPlotChangeDate, PlotTypes.PLOT_LAND),	#iceland
	
	(10, 50) : (constPlotChangeDate, PlotTypes.PLOT_HILLS),	# california enlargement + interior valley
	(10, 49) : (constPlotChangeDate, PlotTypes.PLOT_LAND),
	(10, 48) : (constPlotChangeDate, PlotTypes.PLOT_HILLS),
	(10, 47) : (constPlotChangeDate, PlotTypes.PLOT_HILLS),
	
	(29, 12) : (constPlotChangeDate, PlotTypes.PLOT_HILLS), # santiago
	(6, 59) : (constPlotChangeDate, PlotTypes.PLOT_HILLS), # alaska
}


dFeaturesDict = {
	(35, 54) : (700,  iMud),         # Newfoundland obstacles
	(92, 35) : (constNativeDate, iRainforest),  # Deccan
	(11, 46) : (constNativeDate, iFloodPlains), # California
	(11, 47) : (constNativeDate, iFloodPlains), # California
	(11, 48) : (constNativeDate, iFloodPlains), # California
	
	(10, 50) : (constNativeDate, iForest),	#california forests
	(10, 47) : (constNativeDate, iForest),	#california forests
	
	(107, 44): (constNativeDate, iFloodPlains),
	(106, 44): (constNativeDate, iFloodPlains),
	(105, 45): (constNativeDate, iFloodPlains),
	(53, 54) : (constNativeDate, iFloodPlains),
	(56, 49) : (constNativeDate, iFloodPlains),
	(85, 37) : (constNativeDate, iFloodPlains),
	(56, 46) : (constNativeDate, iFloodPlains),
	(62, 42) : (constNativeDate, iFloodPlains),
	(66, 53) : (constNativeDate, iFloodPlains),
	(58, 47) : (constNativeDate, iFloodPlains),
	(59, 47) : (constNativeDate, iFloodPlains),
	(55, 50) : (constNativeDate, iFloodPlains),
	(62, 53) : (constNativeDate, iFloodPlains),
	(62, 51) : (constNativeDate, iFloodPlains),
	(62, 49) : (constNativeDate, iFloodPlains),
	(59, 54) : (constNativeDate, iFloodPlains),
	(60, 40) : (constNativeDate, iFloodPlains),
	(52, 43) : (constNativeDate, iFloodPlains),
	(51, 43) : (constNativeDate, iFloodPlains),
	(56, 48) : (constNativeDate, iFloodPlains),
	(56, 31) : (constNativeDate, iFloodPlains),
	(55, 44) : (constNativeDate, iFloodPlains),
	(69, 52) : (constNativeDate, iFloodPlains),
	(72, 43) : (constNativeDate, iFloodPlains),
	(69, 30) : (constNativeDate, iFloodPlains),
	(26, 29) : (constNativeDate, iFloodPlains),
	(67, 14) : (constNativeDate, iFloodPlains),
	(38, 22) : (constNativeDate, iFloodPlains),
	(28, 28) : (constNativeDate, iFloodPlains),
	(17, 36) : (constNativeDate, iFloodPlains),
	(27, 29) : (constNativeDate, iFloodPlains),
	(19, 38) : (constNativeDate, iFloodPlains),
	(16, 39) : (constNativeDate, iFloodPlains),
	(73, 54) : (constNativeDate, iFloodPlains),
	(15, 45) : (constNativeDate, iFloodPlains),
	(31, 48) : (constNativeDate, iFloodPlains),		# new york
	(34, 11) : (constNativeDate, iFloodPlains),		# buenos aires
	(52, 44) : (constNativeDate, iFloodPlains),		# madrid
	(41, 18) : (constNativeDate, iFloodPlains),		# rio
	
	(23, 49) : (constNativeDate, iFloodPlains),
	(30, 47) : (constNativeDate, iFloodPlains),
	(30, 48) : (constNativeDate, iFloodPlains),
	(31, 49) : (constNativeDate, iFloodPlains),
	(28, 49) : (constNativeDate, iFloodPlains),
	(113, 45) : (constNativeDate, iFloodPlains),
	(116, 47) : (constNativeDate, iFloodPlains),
	(115, 45) : (constNativeDate, iFloodPlains),
	(109, 46) : (constNativeDate, iFloodPlains),
	(39, 17) : (constNativeDate, iFloodPlains),
	(40, 17) : (constNativeDate, iFloodPlains),
	(39, 18) : (constNativeDate, iFloodPlains),
	(38, 18) : (constNativeDate, iFloodPlains),
	(102, 46) : (constNativeDate, iFloodPlains),
	(106, 43) : (constNativeDate, iFloodPlains),
	(105, 43) : (constNativeDate, iFloodPlains),
	(107, 43) : (constNativeDate, iFloodPlains),
	(80, 40) : (constNativeDate, iFloodPlains),
	(80, 42) : (constNativeDate, iFloodPlains),
	(78, 43) : (constNativeDate, iFloodPlains),
	(88, 49) : (constNativeDate, iFloodPlains),
	(99, 47) : (constNativeDate, iFloodPlains),
	(102, 47) : (constNativeDate, iFloodPlains),
	(100, 44) : (constNativeDate, iFloodPlains),
	(99, 41) : (constNativeDate, iFloodPlains),
	(101, 41) : (constNativeDate, iFloodPlains),
	(101, 39) : (constNativeDate, iFloodPlains),	
	
	(89, 51) : (constNativeDate, iFloodPlains),
	(69, 36) : (constNativeDate, iFloodPlains),
	(71, 28) : (constNativeDate, iFloodPlains),
	(71, 45) : (constNativeDate, iFloodPlains),
	(61, 47) : (constNativeDate, iFloodPlains),
	(101, 33) : (constNativeDate, iFloodPlains),
	(101, 34) : (constNativeDate, iFloodPlains),
	(101, 37) : (constNativeDate, iFloodPlains),
	(92, 32) : (constNativeDate, iFloodPlains),
	(88, 34) : (constNativeDate, iFloodPlains),
	(89, 35) : (constNativeDate, iFloodPlains),
	(79, 43) : (constNativeDate, iFloodPlains),
	(54, 42) : (constNativeDate, iFloodPlains),
	(100, 47) : (constNativeDate, iFloodPlains),
	(101, 42) : (constNativeDate, iFloodPlains),
	(108, 34) : (constNativeDate, iFloodPlains),
	(104, 25) : (constNativeDate, iFloodPlains),
	(106, 25) : (constNativeDate, iFloodPlains),
	(91, 34) : (constNativeDate, iFloodPlains),
	(90, 31) : (constNativeDate, iFloodPlains),
	(92, 32) : (constNativeDate, iFloodPlains),
	(15, 44) : (constNativeDate, iFloodPlains),
	(13, 44) : (constNativeDate, iFloodPlains),
	(60, 50) : (constNativeDate, iFloodPlains),
	(59, 53) : (constNativeDate, iFloodPlains),
	(61, 53) : (constNativeDate, iFloodPlains),
	(56, 50) : (constNativeDate, iFloodPlains),
	(62, 50) : (constNativeDate, iFloodPlains),
	(105, 39) : (constNativeDate, iFloodPlains),
	(102, 42) : (constNativeDate, iFloodPlains),
	(103, 42) : (constNativeDate, iFloodPlains),
	(103, 43) : (constNativeDate, iFloodPlains),
	(55, 46) : (constNativeDate, iFloodPlains),
	(55, 45) : (constNativeDate, iFloodPlains),
	(91, 47) : (constNativeDate, iFloodPlains),
	(89, 50) : (constNativeDate, iFloodPlains),
	(99, 43) : (constNativeDate, iFloodPlains),
	(31, 31) : (constNativeDate, iFloodPlains),
	(53, 53) : (constNativeDate, iFloodPlains),	# London
	(99, 40) : (constNativeDate, iFloodPlains),	# Chengdu
	(60, 46) : (constNativeDate, iFloodPlains),	# Po valley
	
	(93, 50) : (constNativeDate, iOasis),
	(50, 36) : (constNativeDate, iOasis),
	(110, 40) : (constNativeDate, iForest),
	(81, 44) : (constNativeDate, iOasis),		#Tehran
	
	(47, 35) : (constNativeDate, iIslands),		# canaries
	(113, 44) : (constNativeDate, iIslands),
	(112, 43) : (constNativeDate, iIslands),
	(111, 41) : (constNativeDate, iIslands),
	(62, 39) : (constNativeDate, iIslands),
	(64, 42) : (constNativeDate, iIslands),
	(51, 57) : (constNativeDate, iIslands),
	(61, 56) : (constNativeDate, iIslands),
	(65, 57) : (constNativeDate, iIslands),
	(62, 46) : (constNativeDate, iIslands),
	(27, 32) : (constNativeDate, iIslands),
	(109, 39) : (constNativeDate, iIslands),
	(107, 29) : (constNativeDate, iIslands),
	(102, 25) : (constNativeDate, iIslands),
	(97, 29) : (constNativeDate, iIslands),
	(98, 27) : (constNativeDate, iIslands),
	(107, 25) : (constNativeDate, iIslands),
	(29, 33) : (constNativeDate, iIslands),
	(25, 40) : (constNativeDate, iIslands),
	(28, 37) : (constNativeDate, iIslands),
	(30, 47) : (constNativeDate, iIslands),
	(63, 55) : (constNativeDate, iIslands),
	(63, 45) : (constNativeDate, iIslands),
	(69, 40) : (constNativeDate, iIslands),
	(110, 44) : (constNativeDate, iIslands),	#korea
	(113, 42) : (constNativeDate, iIslands),	#kagoshima
	
	(105, 38) : (constNativeDate, iIslands),	#guangzhou
	(106, 37) : (constNativeDate, iIslands),
	
	(23, 34) : (constNativeDate, iRainforest),	#maya
	
	(64, 22) : (constNativeDate, iRainforest),	#kinshasa
	(27, 38) : (constNativeDate, iRainforest),
	(33, 31) : (constNativeDate, iRainforest),
	(35, 29) : (constNativeDate, iRainforest),
	(108, 35) : (constNativeDate, iRainforest),
	(109, 31) : (constNativeDate, iRainforest),
	(108, 30) : (constNativeDate, iRainforest),
	(110, 32) : (constNativeDate, iForest),
	(32, 21) : (constNativeDate, iRainforest),
	(66, 57) : (constNativeDate, iForest),
	(49, 58) : (constNativeDate, iForest),
	(49, 56) : (constNativeDate, iForest),
	(67, 57) : (constNativeDate, iForest),
	(61, 54) : (constNativeDate, iForest),		# germany forest
	(61, 52) : (constNativeDate, iForest),
	(58, 53) : (constNativeDate, iForest),
	(57, 51) : (constNativeDate, iForest),		# france forest
	
	(51, 29) : (constNativeDate, iJungle),		# ghana and congo
	(49, 29) : (constNativeDate, iJungle),
	(54, 28) : (constNativeDate, iJungle),
	(54, 29) : (constNativeDate, iRainforest),
	(55, 28) : (constNativeDate, iJungle),
	(55, 29) : (constNativeDate, iRainforest),
	(56, 28) : (constNativeDate, iRainforest),
	(57, 27) : (constNativeDate, iRainforest),
	(97, 40) : (constNativeDate, iJungle),
	(97, 38) : (constNativeDate, iJungle),
	(64, 19) : (constNativeDate, iMud),
	(66, 20) : (constNativeDate, iJungle),
	(66, 21) : (constNativeDate, iRainforest),
	(63, 18) : (constNativeDate, iRainforest),
	(98, 38) : (constNativeDate, iJungle),
	(112, 26) : (constNativeDate, iJungle),
	(61, 22) : (constNativeDate, iJungle),		# congo jungle
	(53, 28) : (constNativeDate, iJungle),		# ghana
	(53, 29) : (constNativeDate, iRainforest),		# ghana
	
	(60, 23) : (constNativeDate, iRainforest),
	(115, 33) : (constNativeDate, iRainforest),
	(119, 27) : (constNativeDate, iRainforest),
	(122, 19) : (constNativeDate, iRainforest),
	(32, 31) : (constNativeDate, iRainforest),
	(30, 39) : (constNativeDate, iRainforest),
	(27, 36) : (constNativeDate, iRainforest),
	(36, 28) : (constNativeDate, iRainforest),
	(53, 26) : (constNativeDate, iRainforest),	# gold coast
	
	(47, 63) : (constNativeDate,  iMud),         # Iceland has too many cities and too much pop
	(45, 63) : (constNativeDate,  iMud),         # Iceland has too many cities and too much pop
	
	(54, 36) : (constNativeDate, iOasis),		# sahara morocco
	(56, 38) : (constNativeDate, iOasis),		# algiers
	(60, 30) : (constNativeDate, iOasis),		# kano
	(55, 34) : (constNativeDate, iOasis), 		# tamanrasset
	
	(35, 17) : (constNativeDate, iJungle),		# brazil needs more jungle and rainforest
	(33, 15) : (constNativeDate, iJungle),
	(39, 22) : (constNativeDate, iJungle),
	(40, 23) : (constNativeDate, iJungle),
	(33, 29) : (constNativeDate, iJungle),
	(34, 28) : (constNativeDate, iJungle),
	
	(36, 17) : (constNativeDate, iRainforest),
	(37, 28) : (constNativeDate, iRainforest),
	(41, 25) : (constNativeDate, iRainforest),
	(40, 24) : (constNativeDate, iRainforest),
	(42, 24) : (constNativeDate, iRainforest),
	(34, 18) : (constNativeDate, iRainforest),
	
	(104, 37) : (constNativeDate, iRainforest),	# hainan
	
	(109, 51) : (constNativeDate, iMud), 	# vladivostok
	
	(110, 16) : (constNativeDate, iOasis), # alice springs
}

dRemovedFeaturesDict = {
	(67, 30)  : 550,  # Sudan
	(67, 31)  : 550,  # Sudan
	(113, 25) : constNativeDate, # allow settling New Guinea
	(102, 35) : 1350, # open up Vietnam
	(35, 54)  : 1500, # Newfoundland blocker
	(116, 24) : 1500, # Port Moresby
	(82, 47)  : 1600, # Transoxiana
	(83, 46)  : 1600, # Transoxiana
	(85, 49)  : 1600, # Transoxiana
	(114, 51)  : constNativeDate,  # Sapporo
	(56, 35)  : constNativeDate,  # sahara oasis
	(53, 34)  : constNativeDate,  # sahara oasis
	(34, 15)  : constNativeDate,  # asuncion							 
}


@handler("BeginGameTurn")
def createResources():
	for (x, y), iResource in dResources[game.getGameTurn()]:
		createResource(x, y, iResource)


@handler("BeginGameTurn")
def createResourcesBeforeSpawn(iGameTurn):
	for iCiv in dSpawnResources:
		if iGameTurn == year(dBirth[iCiv]) - 1 and data.isCivEnabled(iCiv):
			for (x, y), iResource in dSpawnResources[iCiv]:
				createResource(x, y, iResource)


@handler("BeginGameTurn")
def removeResources():
	for x, y in dRemovedResources[game.getGameTurn()]:
		removeResource(x, y)


@handler("BeginGameTurn")
def createRoutes():
	for tile in dRoutes[game.getGameTurn()]:
		plot(tile).setRouteType(iRouteRoad)


@handler("BeginGameTurn")
def createRoutesBeforeSpawn(iGameTurn):
	for iCiv in dSpawnRoutes:
		if iGameTurn == year(dBirth[iCiv]) - 2 and data.isCivEnabled(iCiv):
			for tile in dSpawnRoutes[iCiv]:
				plot(tile).setRouteType(iRouteRoad)


@handler("BeginGameTurn")
def changePlotType():
	for tile, type in dPlotTypes[game.getGameTurn()]:
		plot(tile).setPlotType(type, True, True)


@handler("BeginGameTurn")
def createFeatures():
	for tile, iFeature in dFeatures[game.getGameTurn()]:
		plot(tile).setFeatureType(iFeature, 0)


@handler("BeginGameTurn")
def removeFeatures(iGameTurn):
	for tile in dRemovedFeatures[game.getGameTurn()]:
		plot(tile).setFeatureType(-1, 0)
		
	if iGameTurn == year(700) and player(iVikings).isHuman():
		plot(41, 58).setFeatureType(-1, 0)
		

# Leoreth: bonus removal alerts by edead
def createResource(iX, iY, iBonus, createTextKey="TXT_KEY_MISC_DISCOVERED_NEW_RESOURCE", removeTextKey="TXT_KEY_MISC_EVENT_RESOURCE_EXHAUSTED"):
	"""Creates a bonus resource and alerts the plot owner"""
	plot = plot_(iX, iY)
	
	iRemovedBonus = plot.getBonusType(-1) # for alert
	
	if iRemovedBonus == iBonus:
		return
	
	plot.setBonusType(iBonus)
			
	if iBonus == -1:
		iImprovement = plot.getImprovementType()
		if iImprovement >= 0:
			if infos.improvement(iImprovement).isImprovementBonusTrade(iRemovedBonus):
				plot.setImprovementType(-1)
		
	iOwner = plot.getOwner()
	if iOwner >= 0: # only show alert to the tile owner
		bWater = plot.isWater()
		city = closestCity(plot, iOwner, same_continent=not bWater, coastal_only=bWater)
		
		if iRemovedBonus >= 0:
			notifyResource(iOwner, city, iX, iY, iRemovedBonus, removeTextKey)
		
		if iBonus >= 0:
			notifyResource(iOwner, city, iX, iY, iBonus, createTextKey)


def notifyResource(iPlayer, city, iX, iY, iBonus, textKey):
	if not city: return
	
	if infos.bonus(iBonus).getTechReveal() == -1 or team(iPlayer).isHasTech(infos.bonus(iBonus).getTechReveal()):
		message(iPlayer, textKey, infos.bonus(iBonus).getText(), city.getName(), event=InterfaceMessageTypes.MESSAGE_TYPE_MINOR_EVENT, button=infos.bonus(iBonus).getButton(), location=(iX, iY))


def removeResource(iX, iY):
	"""Removes a bonus resource and alerts the plot owner"""
	if plot(iX, iY).getBonusType(-1) == -1: return
	createResource(iX, iY, -1)